TO_MICROS_UNITS = 1e6
TO_ANGSTROMS_UNITS = 1e10


class StatisticsName:

    mean = 'mean'
    median = 'median'
    std = 'std'
    min = 'min'
    max = 'max'


class StatisticsRegion:

    full = 'full'
    roi = 'roi'
